/*
 * Copyright (c) 2020 QMTH All Rights Reserved.
 * Created by Deason on 2020-08-12 16:19:32
 */

package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.JsonMapper;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.excel.ExcelWriter;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExportTaskStatus;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExportTaskType;
import cn.com.qmth.examcloud.core.oe.admin.service.*;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.exporttask.ExportTaskInfo;
import cn.com.qmth.examcloud.support.filestorage.FileStorageUtil;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import cn.com.qmth.examcloud.web.filestorage.FileStoragePathEnvInfo;
import cn.com.qmth.examcloud.web.filestorage.YunPathInfo;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * 异步导出相关接口
 */
@Service
public class AsyncExportServiceImpl implements AsyncExportService {

    private static final ExamCloudLog log = ExamCloudLogFactory.getLog(AsyncExportServiceImpl.class);

    private static final String TASK_EXPORT_DIR = "task_export";

    @Autowired
    private ExportTaskService exportTaskService;

    @Autowired
    private ExamRecordService examRecordService;

    @Autowired
    private ExamCaptureService examCaptureService;

    @Autowired
    private ExamScoreService examScoreService;

    @Autowired
    private SystemProperties systemConfig;

    @Override
    public void exportExamRecordDetails(String jsonParams) {
        ExamRecordQuery req = new JsonMapper().fromJson(jsonParams, ExamRecordQuery.class);
        Check.isNull(req, "请求参数不能为空！");
        Check.isNull(req.getRootOrgId(), "学校ID不能为空！");
        Check.isNull(req.getExamId(), "考试ID不能为空！");
        Check.isNull(req.getCreator(), "创建人ID不能为空！");

        boolean existTask = exportTaskService.existUnFinishTask(ExportTaskType.EXAM_DETAIL, req.getRootOrgId(), req.getExamId());
        if (existTask) {
            log.warn("exportExamRecordDetails has existed, please waiting...");
            throw new StatusException("500S01", "当前考试尚有未完成的导出任务，请稍后再导！");
        }

        // 创建导出任务
        ExportTaskInfo task = new ExportTaskInfo();
        task.setRootOrgId(req.getRootOrgId());
        task.setExamId(req.getExamId());
        task.setType(ExportTaskType.EXAM_DETAIL);
        task.setStatus(ExportTaskStatus.EXPORTING);
        task.setCreator(req.getCreator());
        Long taskId = exportTaskService.addExportTask(task);

        this.asyncExportExamRecordDetails(taskId, req);
    }

    @Async
    public void asyncExportExamRecordDetails(Long taskId, ExamRecordQuery req) {
        List<ExamRecordInfo> examRecords;
        try {
            examRecords = examRecordService.getExamRecordDetailList(req);

            if (CollectionUtils.isEmpty(examRecords)) {
                exportTaskService.updateExportTaskStatus(taskId, ExportTaskStatus.ERROR, "当前条件暂无数据，任务终止");
                return;
            }

            String examType = examRecords.get(0).getExamType();
            for (ExamRecordInfo examRecordInfo : examRecords) {
                if (ExamType.ONLINE.name().equals(examType) || ExamType.ONLINE_HOMEWORK.name().equals(examType)) {
                    examRecordInfo.setVirtualCameraNames(examCaptureService.getVirtualCameraNames(examRecordInfo.getDataId()));
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage());
            if (e instanceof StatusException) {
                exportTaskService.updateExportTaskStatus(taskId, ExportTaskStatus.ERROR, ((StatusException) e).getDesc());
            } else {
                exportTaskService.updateExportTaskStatus(taskId, ExportTaskStatus.ERROR, "导出数据异常");
            }
            return;
        }

        // 导出文件的存储路径
        final String filePath = String.format("/%s/%s/%s/e_record_detail_%s.xlsx", TASK_EXPORT_DIR, req.getRootOrgId(), dateDir(), randomUUID());
        String tempFilePath = systemConfig.getTempDataDir() + File.separator + filePath;
        File tempFile = new File(tempFilePath);
        try {
            tempFile.getParentFile().mkdirs();
            tempFile.createNewFile();
        } catch (IOException e) {
            log.error(e.getMessage());
            exportTaskService.updateExportTaskStatus(taskId, ExportTaskStatus.ERROR, "生成导出文件异常");
            return;
        }

        try (FileOutputStream out = new FileOutputStream(tempFile);) {
            ExcelWriter excelWriter = new ExcelWriter(ExamRecordInfo.class, "考试明细列表");
            excelWriter.write(examRecords, out);

            // 上传至文件服务器
            FileStoragePathEnvInfo env = new FileStoragePathEnvInfo();
            env.setRootOrgId(String.valueOf(req.getRootOrgId()));
            env.setRelativePath(filePath);
            YunPathInfo oss = FileStorageUtil.saveFile(TASK_EXPORT_DIR, env, tempFile, null);

            log.info("asyncExportExamRecordDetails finished... " + oss.getRelativePath());
            exportTaskService.updateExportTaskStatus(taskId, ExportTaskStatus.SUCCESS, null, oss.getRelativePath());
        } catch (Exception e) {
            log.info("asyncExportExamRecordDetails error... " + e.getMessage());
            exportTaskService.updateExportTaskStatus(taskId, ExportTaskStatus.ERROR, "上传至文件服务器异常");
        } finally {
            FileUtils.deleteQuietly(tempFile);
        }
    }

    @Override
    public void exportExamScoreStatistics(String jsonParams) {
        ExamScoreQuery req = new JsonMapper().fromJson(jsonParams, ExamScoreQuery.class);
        Check.isNull(req, "请求参数不能为空！");
        Check.isNull(req.getRootOrgId(), "学校ID不能为空！");
        Check.isNull(req.getExamId(), "考试ID不能为空！");
        Check.isNull(req.getCreator(), "创建人ID不能为空！");

        boolean existTask = exportTaskService.existUnFinishTask(ExportTaskType.SCORE_STATISTIC, req.getRootOrgId(), req.getExamId());
        if (existTask) {
            log.warn("exportExamScoreStatistics has existed, please waiting...");
            throw new StatusException("500S02", "当前考试尚有未完成的导出任务，请稍后再导！");
        }

        // 创建导出任务
        ExportTaskInfo task = new ExportTaskInfo();
        task.setRootOrgId(req.getRootOrgId());
        task.setExamId(req.getExamId());
        task.setType(ExportTaskType.SCORE_STATISTIC);
        task.setStatus(ExportTaskStatus.EXPORTING);
        task.setCreator(req.getCreator());
        Long taskId = exportTaskService.addExportTask(task);

        this.asyncExportExamScoreStatistics(taskId, req);
    }

    @Async
    public void asyncExportExamScoreStatistics(Long taskId, ExamScoreQuery req) {
        List<ExamScoreInfo> examScores;
        try {
            examScores = examScoreService.exportExamScoreList(req);

            if (CollectionUtils.isEmpty(examScores)) {
                exportTaskService.updateExportTaskStatus(taskId, ExportTaskStatus.ERROR, "当前条件暂无数据，任务终止");
                return;
            }
        } catch (Exception e) {
            log.error(e.getMessage());
            if (e instanceof StatusException) {
                exportTaskService.updateExportTaskStatus(taskId, ExportTaskStatus.ERROR, ((StatusException) e).getDesc());
            } else {
                exportTaskService.updateExportTaskStatus(taskId, ExportTaskStatus.ERROR, "导出数据异常");
            }
            return;
        }

        // 导出文件的存储路径
        final String filePath = String.format("/%s/%s/%s/e_score_statistic_%s.xlsx", TASK_EXPORT_DIR, req.getRootOrgId(), dateDir(), randomUUID());
        String tempFilePath = systemConfig.getTempDataDir() + File.separator + filePath;
        File tempFile = new File(tempFilePath);
        try {
            tempFile.getParentFile().mkdirs();
            tempFile.createNewFile();
        } catch (IOException e) {
            log.error(e.getMessage());
            exportTaskService.updateExportTaskStatus(taskId, ExportTaskStatus.ERROR, "生成导出文件异常");
            return;
        }

        try (FileOutputStream out = new FileOutputStream(tempFile);) {
            ExcelWriter excelWriter = new ExcelWriter(ExamScoreInfo.class, "成绩统计列表");
            excelWriter.write(examScores, out);

            // 上传至文件服务器
            FileStoragePathEnvInfo env = new FileStoragePathEnvInfo();
            env.setRootOrgId(String.valueOf(req.getRootOrgId()));
            env.setRelativePath(filePath);
            YunPathInfo oss = FileStorageUtil.saveFile(TASK_EXPORT_DIR, env, tempFile, null);

            log.info("asyncExportExamScoreStatistics finished... " + oss.getRelativePath());
            exportTaskService.updateExportTaskStatus(taskId, ExportTaskStatus.SUCCESS, null, oss.getRelativePath());
        } catch (Exception e) {
            log.info("asyncExportExamScoreStatistics error... " + e.getMessage());
            exportTaskService.updateExportTaskStatus(taskId, ExportTaskStatus.ERROR, "上传至文件服务器异常");
        } finally {
            FileUtils.deleteQuietly(tempFile);
        }
    }

    private String randomUUID() {
        return UUID.randomUUID().toString().replace("-", "");
    }

    private String dateDir() {
        final String pattern = "yyyyMM";
        return new SimpleDateFormat(pattern).format(new Date());
    }

}
